<?php

declare(strict_types=1);

namespace Drupal\jsonapi_request_logger\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Class that records detailed information of each JSON:API HTTP request.
 */
final class RequestLoggerSubscriber implements EventSubscriberInterface {

  /**
   * Config factory interface for accessing configuration settings.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  private ConfigFactoryInterface $configFactory;

  /**
   * Logger interface for logging messages.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private LoggerInterface $logger;

  /**
   * Constructs the RequestLoggerSubscriber.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  public function __construct(ConfigFactoryInterface $configFactory, LoggerInterface $logger) {
    $this->configFactory = $configFactory;
    $this->logger = $logger;
  }

  /**
   * Handles the kernel request event.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The request event.
   */
  public function onKernelRequest(RequestEvent $event): void {
    $config = $this->configFactory->get('jsonapi_request_logger.settings');

    if (!$config->get('enable')) {
      return;
    }

    $request = $event->getRequest();
    if ($request->getRequestFormat() !== 'api_json') {
      return;
    }

    if ($this->isExcludedRoute($request->getPathInfo(), $config->get('exclude_routes'))) {
      return;
    }

    if ($this->isMethodNotAllowed($request->getMethod(), $config->get('request_methods'))) {
      return;
    }

    if ($config->get('omit_empty_body') && empty($request->getContent())) {
      return;
    }

    $context = $this->buildRequestContext($request);
    $request->attributes->set('jsonapi_request_log_context', $context);
  }

  /**
   * Handles the kernel response event.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The response event.
   */
  public function onKernelResponse(ResponseEvent $event): void {
    $config = $this->configFactory->get('jsonapi_request_logger.settings');

    if (!$config->get('enable')) {
      return;
    }

    $request = $event->getRequest();
    $context = $request->attributes->get('jsonapi_request_log_context');

    if ($context === null) {
      return;
    }

    $context['{status_code}'] = $event->getResponse()->getStatusCode();
    $log_format = $config->get('log_format') ?? 'json';
    $log_template = $config->get('log_output_template');
    $formatted_message = $this->formatLogMessage($context, $log_template, $log_format);

    $this->logger->info($formatted_message);
  }

  /**
   * Determines if the current route is excluded from logging.
   *
   * @param string $current_path
   *   The current request path.
   * @param string $exclude_routes
   *   A list of excluded routes patterns.
   *
   * @return bool
   *   TRUE if the route is excluded, FALSE otherwise.
   */
  private function isExcludedRoute(string $current_path, string $exclude_routes): bool {
    $routes = preg_split('/\r\n|\r|\n/', $exclude_routes);

    foreach ($routes as $pattern) {
      if (substr($pattern, -1) === '*') {
        $pattern = rtrim($pattern, '*');
        if (strpos($current_path, $pattern) === 0) {
          return TRUE;
        }
      }
      elseif ($pattern === $current_path) {
        return TRUE;
      }
    }

    return FALSE;
  }


  /**
   * Checks if the HTTP method is allowed for logging.
   *
   * @param string $method
   *   The HTTP method of the request.
   * @param array $allowed_methods
   *   An array of allowed HTTP methods.
   *
   * @return bool
   *   TRUE if the method is not allowed, FALSE otherwise.
   */
  private function isMethodNotAllowed(string $method, array $allowed_methods): bool {
    return !empty($allowed_methods) && !in_array($method, $allowed_methods, true);
  }

  /**
   * Builds the context array for the request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The HTTP request.
   *
   * @return array
   *   The context array for logging.
   */
  private function buildRequestContext($request): array {
    return [
      '{method}' => $request->getMethod(),
      '{url}' => $request->getUri(),
      '{headers}' => json_encode($request->headers->all(), JSON_PRETTY_PRINT),
      '{query}' => $request->getQueryString(),
      '{content}' => $request->getContent(),
    ];
  }

  /**
   * Formats the log message based on the specified format.
   *
   * @param array $context
   *   The context array with request details.
   * @param string $template
   *   The log message template.
   * @param string $format
   *   The format type for the log message.
   *
   * @return string
   *   The formatted log message.
   */
  private function formatLogMessage(array $context, string $template, string $format): string {
    switch ($format) {
      case 'json':
        return $this->formatAsJson($context, $template);

      default:
        return strtr($template, $context);
    }
  }

  /**
   * Formats the log message as JSON.
   *
   * @param array $context
   *   The context array with request details.
   * @param string $template
   *   The log message template.
   *
   * @return string
   *   The formatted JSON log message.
   */
  private function formatAsJson(array $context, string $template): string {
    $ordered_context = [];
    preg_match_all('/\{(.*?)\}/', $template, $matches);

    foreach ($matches[0] as $token) {
      $ordered_context[trim($token, '{}')] = $context[$token] ?? '';
    }

    return json_encode($ordered_context, JSON_PRETTY_PRINT);
  }

  /**
   * Returns an array of subscribed events and their corresponding methods.
   *
   * @return array
   *   The array of events with their methods.
   */
  public static function getSubscribedEvents(): array {
    return [
      KernelEvents::REQUEST => ['onKernelRequest'],
      KernelEvents::RESPONSE => ['onKernelResponse'],
    ];
  }
}
