<?php

namespace Drupal\jsonapi_request_logger\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure jsonapi_request_logger settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'jsonapi_request_logger_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['jsonapi_request_logger.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('jsonapi_request_logger.settings');

    // Select for log format.
    $form['log_format'] = [
      '#type' => 'select',
      '#title' => $this->t('Log format'),
      '#description' => $this->t('Choose the format for logging: JSON.'),
      '#options' => [
        'json' => $this->t('JSON'),
      ],
      '#default_value' => $config->get('log_format') ?? 'json',
    ];

    // Checkbox to enable or disable logging.
    $form['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable JSON:API request logging'),
      '#description' => $this->t('Caution. This type of logging will log all incoming requests and could quickly corrupt the log message table. Be careful.'),
      '#default_value' => $config->get('enable'),
    ];

    // Checkbox to omit requests with an empty body.
    $form['omit_empty_body'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Omit requests with empty body'),
      '#description' => $this->t('If enabled, requests with an empty body will not be logged.'),
      '#default_value' => $config->get('omit_empty_body'),
    ];

    // Multi-selection for HTTP request methods.
    $form['request_methods'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Request methods to log'),
      '#description' => $this->t('Select the HTTP methods that should be logged. If none are selected, all methods will be logged.'),
      '#options' => [
        'GET' => $this->t('GET'),
        'POST' => $this->t('POST'),
        'PATCH' => $this->t('PATCH'),
        'DELETE' => $this->t('DELETE'),
      ],
      '#default_value' => $config->get('request_methods') ?? [],
    ];

    // Textarea for excluded routes.
    $form['exclude_routes'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Exclude routes'),
      '#description' => $this->t('Enter routes to exclude from logging. Use * as a wildcard to match multiple routes.'),
      '#default_value' => $config->get('exclude_routes'),
    ];

    // Textfield for log output template with token explanations.
    $form['log_output_template'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Log output template'),
      '#description' => $this->t('Customize the log output using the following tokens: {method}, {url}, {status_code}, {headers}, {query}, {content}.'),
      '#default_value' => $config->get('log_output_template'),
      '#size' => 100,
    ];

    // Collapsible details section explaining the replacement tokens.
    $form['token_details'] = [
      '#type' => 'details',
      '#title' => $this->t('Available replacement tokens'),
      '#description' => $this->t('The following tokens can be used in the log output template to customize the logged information:'),
      '#open' => FALSE, // Details is collapsed by default.
      'token_explanation' => [
        '#markup' => '
          <ul>
            <li><strong>{method}</strong>: The HTTP method of the request (e.g., GET, POST).</li>
            <li><strong>{url}</strong>: The full URL of the request.</li>
            <li><strong>{status_code}</strong>: The HTTP status code of the response.</li>
            <li><strong>{headers}</strong>: The request headers in JSON format.</li>
            <li><strong>{query}</strong>: The query string of the request.</li>
            <li><strong>{content}</strong>: The body of the request (if available).</li>
          </ul>
        ',
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('jsonapi_request_logger.settings')
      ->set('enable', $form_state->getValue('enable'))
      ->set('exclude_routes', $form_state->getValue('exclude_routes'))
      ->set('omit_empty_body', $form_state->getValue('omit_empty_body'))
      ->set('request_methods', array_filter($form_state->getValue('request_methods')))
      ->set('log_output_template', $form_state->getValue('log_output_template'))
      ->set('log_format', $form_state->getValue('log_format'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
